/*  */ /* <-- This control code makes my KX-P1180 printer in condensed mode */
/*  FDC v1.00                                      date : 1-03-90


    The following test material references comes from

    1) the Users Guide for the Floppy Disk Controler_Plus
       (A Western Digital Clone),

    2) the 1989 National Semiconductor Mass Storage Hand Book
       (referencing the Floppy Disk Controller_Plus2
        or the NS DP8473 Floppy Diskette Controller),

    3) the NEC 1981 Catalog for Microcomputers
       or the NEC PD765.

    4) and the IBM Technical Reference Manual for the PC.

///////////////////////////////////////////////////////////////////

   I O  Port Addresses for the Floppy Disk Controler(s)

*/

#define Digital_Output_R    0x03f2      /* aka Drive control Register */
#define Main_Status_R       0x03f4      /* Main Status Register */
#define Diskette_Data_R     0x03f5      /* Diskette Data Transfer Register */

/* This register is valid for the FDC_plus and the NS FDC_Plus 2 */
#define Digital_Input_R     0x03f7      /* aka Data Rate Register */

/* This register is only valid in the NS FDC_Plus2  */
#define Diskette_Line_Bit   0x03f8      /*  Only bit 7 is used for floppies
                                            bit 7 (seven) = 1, if line change */

/*
  ///////////////////////////////////////////////////////////////////

   I O  Bit Register Mapping for differences in the Floppy Disk Controler(s)

*/

/* note: The different structure(s) may are very similar
         and the Reserved BITS are used by the NEC PD765
         NS FDC_Plus 2, as indicated.
*/

struct
 {unsigned int Reserv3        : 1;  /* Enable Motor for drive 4 */
  unsigned int Reserv2        : 1;  /*    "    "     "    "   3 */
  unsigned int DrvBMotorEnb   : 1;  /* 0 == off */
  unsigned int DrvAMotorEnb   : 1;  /* 0 == off */
  unsigned int EnbDiskInt_DMA : 1;  /* 0 == disable */
  unsigned int ResetCntlr     : 1;  /* 0 == Reset */
  unsigned int Reserv1        : 1;  /* Drive Select Bits for Drive 3 & 4 */
  unsigned int Disk_Select    : 1;  /* 0 == DriveA */
 } Dig_Out_Reg;


/* note: The different structure(s) may are very similar
         and the Reserved BITS are not used by either.

         FDC_Plus uses bits 0 & 1 as input(s) when writing
         and the other indicated bits when reading.
         The Type returned when reading is not documented.
*/

struct
 {unsigned int DisketteChng   : 1;  /* used only by the FDC_Plus,
                                       1 == line changed */
  unsigned int Reserv3        : 1;
  unsigned int Reserv2        : 1;
  unsigned int Reserv1        : 1;
  unsigned int Type2          : 1;  /* used only by the FDC_Plus */
  unsigned int Type1          : 1;  /* used only by the FDC_Plus */
  unsigned int SelRateOrType2 : 1;  /* Rate Select listed below */
  unsigned int SelRateOrType1 : 1;  /*  bits  rate       controller */
 } Dig_In_Reg;                      /*  ----  --------   ------------
                                        00 == 500 kbps   both
                                        01 == 300 kbps    "
                                        10 == 250 kbps    "
                                        11 == 1000 kbps  FDC_Plus2 only
                                    */

/*
 ///////////////////////////////////////////////////////////////////

        P C     B I O S    Data Segment  as per

        "The IBM Technical Reference for the PC".
         (Listed data is not complete)
*/


/*

Segement Base Address

*/
const unsigned int  BiosDataSeg   =  0x40;


/*

Offsets to the BIOS Data Segement Base Address

*/
const unsigned int  InHardware    =  0x10,
                    POST          =  0x11,
                    BaseMem       =  0x12,
                    KeyBdStatus   =  0x17;   /* two bytes have different data */

/* Floppy Diskettes */
const unsigned char DsktRecalStatus    =  0x3e,
                    DsktMotorStatus    =  0x3f,
                    DsktMotorOffTime   =  0x40,  /* and time-out count
                                                    in lower nibble*/
                    DsktLastOpStatus   =  0x41,
                    DsktStatusCmd      =  0x42;  /* seven bytes here ??
                                                    all references to date
                                                    say there is seven bytes
                                                 */
/* Fix Disk(s) */
const unsigned char FixDskOpStatus     =  0x74,
                    FixDskNumberOf     =  0x75,
                    FixDskCntrlByte    =  0x76,
                    FixDskIOPortAdd    =  0x76;   /* Offset of Port Address */

